/* $Id: tsconfig.c,v 1.5 2003/04/15 00:47:06 chrisf Exp $ */

/*
TreoScreens - Select default screen for phone etc.
Copyright (C) 2003 Chris Faherty

This program is free software; you can redistribute it and/or
modify it under the terms of the GNU General Public License
as published by the Free Software Foundation; either version 2
of the License, or (at your option) any later version.

This program is distributed in the hope that it will be useful,
but WITHOUT ANY WARRANTY; without even the implied warranty of
MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
GNU General Public License for more details.

You should have received a copy of the GNU General Public License
along with this program; if not, write to the Free Software
Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.
*/

#include <PalmOS.h>
#include <Handspring/HsExt.h>
#include <Handspring/HsAppLaunchCmd.h>
#include "tsprefs.h"
#include "tsconfigRsc.h"
#include "tsconfig.h"

/* globals */
DmOpenRef LauncherDB;

/* prefs */
struct sprefs prefs;

Boolean MainFormHandleEvent(EventPtr e)
{
    Boolean handled;
    FormPtr frm;
    ListPtr list;
    ControlPtr ctl;
    Int16 i;
    char *p;

    handled = false;

    switch (e->eType) {
    case frmOpenEvent:
        frm = FrmGetActiveForm();
        FrmDrawForm(frm);

        /* set initial phoneapplaunch setting */

        list = FrmGetObjectPtr(frm,
            FrmGetObjectIndex(frm, listID_mainphoneapplaunch));
        ctl = FrmGetObjectPtr(frm,
            FrmGetObjectIndex(frm, poptrigID_mainphoneapplaunch));
        i = prefs.phoneapplaunch;
        CtlSetLabel(ctl, LstGetSelectionText(list, i));
        LstSetSelection(list, i);

        /* launcher categories */

        ctl = FrmGetObjectPtr(frm,
            FrmGetObjectIndex(frm, poptrigID_mainlaunch1));
        p = (char *)CtlGetLabel(ctl);
        CategoryGetName(LauncherDB, prefs.launch1cat, p);
        CtlSetLabel(ctl, p);

        ctl = FrmGetObjectPtr(frm,
            FrmGetObjectIndex(frm, poptrigID_mainlaunch2));
        p = (char *)CtlGetLabel(ctl);
        CategoryGetName(LauncherDB, prefs.launch2cat, p);
        CtlSetLabel(ctl, p);

        ctl = FrmGetObjectPtr(frm,
            FrmGetObjectIndex(frm, poptrigID_mainlaunch3));
        p = (char *)CtlGetLabel(ctl);
        CategoryGetName(LauncherDB, prefs.launch3cat, p);
        CtlSetLabel(ctl, p);

        handled = true;
        break;

    case frmUpdateEvent:
        frm = FrmGetActiveForm();
        FrmDrawForm(frm);
        handled = true;
        break;

    case menuEvent:
        MenuEraseStatus(NULL);
        switch (e->data.menu.itemID) {
        case mitemID_about:
            FrmPopupForm(AboutForm);
            break;
        }
       handled = true;
       break;

    case ctlSelectEvent:
        switch (e->data.ctlSelect.controlID) {
        case poptrigID_mainlaunch1:
            prefs.launch1cat = SelectCategoryPopup(LauncherDB,
                poptrigID_mainlaunch1, listID_mainlaunch1, prefs.launch1cat);
            SavePrefs(&prefs);
            handled = true;
            break;
        case poptrigID_mainlaunch2:
            prefs.launch2cat = SelectCategoryPopup(LauncherDB,
                poptrigID_mainlaunch2, listID_mainlaunch2, prefs.launch2cat);
            SavePrefs(&prefs);
            handled = true;
            break;
        case poptrigID_mainlaunch3:
            prefs.launch3cat = SelectCategoryPopup(LauncherDB,
                poptrigID_mainlaunch3, listID_mainlaunch3, prefs.launch3cat);
            SavePrefs(&prefs);
            handled = true;
            break;
        }
        break;

    case popSelectEvent:
        i = e->data.popSelect.selection;
        if (i < 0) break;
        switch (e->data.ctlSelect.controlID) {
        case poptrigID_mainphoneapplaunch:
            prefs.phoneapplaunch = i;
            SavePrefs(&prefs);
            break;
        }
        break;

    default:
        break;
    }

    return handled;
}

Boolean AboutFormHandleEvent(EventPtr e)
{
    Boolean handled;
    FormPtr frm;

    handled = false;

    switch (e->eType) {
    case frmOpenEvent:
        frm = FrmGetActiveForm();
        FrmDrawForm(frm);
        handled = true;
        break;

    case menuEvent:
        MenuEraseStatus(NULL);
        switch (e->data.menu.itemID) {
        }
        handled = true;
        break;

    case ctlSelectEvent:
        switch (e->data.ctlSelect.controlID) {
        case buttonID_aboutok:
            FrmReturnToForm(0);
            handled = true;
            break;
        case buttonID_aboutcontrib:
            FrmPopupForm(ContribForm);
            handled = true;
            break;
        }
        break;

    default:
        break;
    }

    return handled;
}

Boolean ContribFormHandleEvent(EventPtr e)
{
    Boolean handled;
    FormPtr frm;
    ListPtr lst;

    handled = false;

    switch (e->eType) {
    case frmOpenEvent:
        frm = FrmGetActiveForm();
        lst = FrmGetObjectPtr(frm, FrmGetObjectIndex(frm, listID_contrib));
        /* turn off the highlight */
        LstSetSelection(lst, -1);
        FrmDrawForm(frm);
        handled = true;
        break;

    case menuEvent:
        MenuEraseStatus(NULL);
        switch (e->data.menu.itemID) {
        }
        handled = true;
        break;

    case ctlSelectEvent:
        switch (e->data.ctlSelect.controlID) {
        case buttonID_contribok:
            FrmReturnToForm(0);
            handled = true;
            break;
        }
        break;

    case keyDownEvent:
        if (e->data.keyDown.chr == pageUpChr) {
            frm = FrmGetActiveForm();
            lst = FrmGetObjectPtr(frm, FrmGetObjectIndex(frm,
                listID_contrib));
            LstScrollList(lst, winUp, LstGetVisibleItems(lst));
        } else if (e->data.keyDown.chr == pageDownChr) {
            frm = FrmGetActiveForm();
            lst = FrmGetObjectPtr(frm, FrmGetObjectIndex(frm,
                listID_contrib));
            LstScrollList(lst, winDown, LstGetVisibleItems(lst));
        }
        break;

    default:
        break;
    }

    return handled;
}

/*
 * Returns field that has the focus, if any, including in embedded tables.
 */
FieldPtr GetFocusObjectPtr(void)
{
    FormPtr frm;
    UInt16 focus;
    FormObjectKind objType;

    frm = FrmGetActiveForm();
    focus = FrmGetFocus(frm);
    if (focus == noFocus) return NULL;
    objType = FrmGetObjectType(frm, focus);
    if (objType == frmFieldObj) return FrmGetObjectPtr(frm, focus);
    else if (objType == frmTableObj)
        return TblGetCurrentField(FrmGetObjectPtr(frm, focus));
    return NULL;
}

Boolean FormHandleMenuEvent(UInt16 menuID)
{
    FieldPtr fld;

    switch (menuID) {
    case sysEditMenuUndoCmd:
    case sysEditMenuCutCmd:
    case sysEditMenuCopyCmd:
    case sysEditMenuPasteCmd:
    case sysEditMenuSelectAllCmd:
        fld = GetFocusObjectPtr();
        if (!fld) return false;
        if (menuID == sysEditMenuUndoCmd) FldUndo(fld);
        else if (menuID == sysEditMenuCutCmd) FldCut(fld);
        else if (menuID == sysEditMenuCopyCmd) FldCopy(fld);
        else if (menuID == sysEditMenuPasteCmd) FldPaste(fld);
        else if (menuID == sysEditMenuSelectAllCmd)
            FldSetSelection (fld, 0, FldGetTextLength (fld));
        return true;
    case sysEditMenuKeyboardCmd:
        SysKeyboardDialog(kbdDefault);
        return true;
    case sysEditMenuGraffitiCmd:
        SysGraffitiReferenceDialog(referenceDefault);
        return true;
    }
    return false;
}

Boolean ApplicationHandleEvent(EventPtr e)
{
    FormPtr frm;
    UInt16 formId;
    Boolean handled;

    handled = false;
    if (e->eType == frmLoadEvent) {
        formId = e->data.frmLoad.formID;
        frm = FrmInitForm(formId);
        FrmSetActiveForm(frm);

        switch(formId) {
        case MainForm:
            FrmSetEventHandler(frm, MainFormHandleEvent);
            break;
        case AboutForm:
            FrmSetEventHandler(frm, AboutFormHandleEvent);
            break;
        case ContribForm:
            FrmSetEventHandler(frm, ContribFormHandleEvent);
            break;
        }
        handled = true;
    }

    return handled;
}

/* Get preferences, open (or create) app database */
UInt16 StartApplication(void)
{
    UInt32 tui32;

    LoadPrefs(&prefs);

    /* determine if we have handspring extensions */
    if (FtrGet(hsFtrCreator, hsFtrIDVersion, &tui32)) return 1;

    if (!OpenLauncherDB(&LauncherDB)) return 1;

    FrmGotoForm(MainForm);
    return 0;
}

/* Save preferences, close forms, close app database */
void StopApplication(void)
{
    FrmSaveAllForms();
    FrmCloseAllForms();
    CloseDatabase(LauncherDB);
}

Boolean OpenLauncherDB(DmOpenRef *db)
{
    /* Set these to zero in case any of the opens fail */
    *db = 0;

    /* Open our database */
    *db = DmOpenDatabaseByTypeCreator('DATA', 'lnch', dmModeReadOnly);

    if (!*db) return false;

    return true;
}

void CloseDatabase(DmOpenRef db)
{
    if (db) {
        DmCloseDatabase(db);
    }
}

/* The main event loop */
void EventLoop(void)
{
    Err err;
    EventType e;
    Int32 eventTimeout;

    eventTimeout = evtWaitForever;

    do {

    EvtGetEvent(&e, eventTimeout);

    if (!SysHandleEvent(&e)) {
        if (!MenuHandleEvent(NULL, &e, &err)) {
            if (!ApplicationHandleEvent(&e)) {
                FrmDispatchEvent(&e);
            }
        }
    }

    } while (e.eType != appStopEvent);

}

UInt8 SelectCategoryPopup(DmOpenRef dbP, UInt16 trigID, UInt16 listID,
    UInt8 cat)
{
    Int16 curSelection;
    Int16 newSelection;
    ListPtr lst;
    ControlPtr ctl;
    char *name;
    RectangleType r, r2;
	FormPtr frm;
    char *p;
    UInt8 newcat=cat;
	
    frm = FrmGetActiveForm();
    lst = FrmGetObjectPtr(frm, FrmGetObjectIndex(frm, listID));
    ctl = FrmGetObjectPtr(frm, FrmGetObjectIndex(frm, trigID));

    LstSetPosition(lst, 0, 0);
    
    /* Create a list of categories. */
    CategoryCreateList(dbP, lst, cat, true, true, 1, 0, true);

    /* Position the list. */
    FrmGetObjectBounds(frm, FrmGetObjectIndex(frm, trigID), &r);
    FrmGetObjectBounds(frm, FrmGetObjectIndex(frm, listID), &r2);
    LstSetPosition(lst, r.topLeft.x, r.topLeft.y);

    /* Display the category list. */
    curSelection = LstGetSelection(lst);
    newSelection = LstPopupList(lst);

    /* Was a new category selected? */
    if ((newSelection != curSelection) && (newSelection >= 0)) {
        if ((name = LstGetSelectionText(lst, newSelection))) {
            newcat = CategoryFind(dbP, name);
            /* copy category name into the trigger */
            p = (char *)CtlGetLabel(ctl);
            /* the trigger is 32 chars, which is max category name */
            StrCopy(p, name);
            CtlSetLabel(ctl, p);
        }
    }

    CategoryFreeList(dbP, lst, false, false);
    return newcat;
}

/*
 * The program starts here.
 */
UInt32 PilotMain(UInt16 cmd, void *cmdPBP, UInt16 launchFlags)
{
    Err err;
    UInt32 romVersion;
    UInt32 theInstalledRAM; 

    switch (cmd) {
    case sysAppLaunchCmdNormalLaunch:
        /*
         * This ROM version test comes from Palm's Address application.
         * Thankfully too because I would hate to guess at this kind of
         * stuff and make some poor guy's Pilot blow up.
         */
        FtrGet(sysFtrCreator, sysFtrNumROMVersion, &romVersion);
        /*
         * Need at least a PalmPilot Pro because of the TCP/IP.
         */
        if (romVersion < 0x03000000) {
            if ((launchFlags &
                (sysAppLaunchFlagNewGlobals | sysAppLaunchFlagUIApp)) ==
                (sysAppLaunchFlagNewGlobals | sysAppLaunchFlagUIApp)) {
                /*
                 * Pilot 1.0 will continuously relaunch this app unless we
                 * switch to another safe one.
                 */
                if (romVersion < 0x02000000) {
                    FrmAlert(RomIncompatibleAlert);
                    AppLaunchWithCommand(sysFileCDefaultApp,
                        sysAppLaunchCmdNormalLaunch, NULL);
                }
                err = MemCardInfo( 0, NULL, NULL, NULL, NULL, NULL,
                    &theInstalledRAM, NULL); 
                if (!err && (theInstalledRAM == 524288)) { 
                    /*
                     * We are running on a PalmPilot Personal unit
                     */
                    FrmAlert(RomIncompatibleAlert);
                    return sysErrRomIncompatible;
                }
            }
        }
        err = StartApplication();
        if (err) return err;
        EventLoop();
        StopApplication();

        break;

    default:
        return sysErrParamErr;
    }

    return 0;
}
