/* $Id: tslaunch.c,v 1.7 2003/04/15 02:59:43 chrisf Exp $ */

/*
TreoScreens - Select default screen for phone etc.
Copyright (C) 2003 Chris Faherty

This program is free software; you can redistribute it and/or
modify it under the terms of the GNU General Public License
as published by the Free Software Foundation; either version 2
of the License, or (at your option) any later version.

This program is distributed in the hope that it will be useful,
but WITHOUT ANY WARRANTY; without even the implied warranty of
MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
GNU General Public License for more details.

You should have received a copy of the GNU General Public License
along with this program; if not, write to the Free Software
Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.
*/

#include <PalmOS.h>
#include <Handspring/HsExt.h>
#include <Handspring/HsAppLaunchCmd.h>
#include "tsprefs.h"
#include "tsconfig.h"
#include "tslaunchRsc.h"
#include "tslaunch.h"

/* globals */

/* prefs */
struct sprefs prefs;

UInt32 GetCreatorID(void)
{
    UInt16 nCard;
    LocalID LocalDB;
    UInt32 nCreatorID;

    ErrFatalDisplayIf(SysCurAppDatabase(&nCard, &LocalDB),
        "Could not get current app database.");
    ErrFatalDisplayIf(DmDatabaseInfo(nCard, LocalDB, 0, 0, 0, 0,
        0, 0, 0, 0, 0, 0, &nCreatorID),
        "Could not get app database info, looking for creator ID");

    return nCreatorID;
}

UInt8 get_proper_category(void)
{
    UInt8 cat;
    /*
     * There are several apps built with this same code, except that each
     * has a separate creator id.  Tsc3=launch1 Tsc4=launch2 etc.  I need to
     * associate these with the launch prefs I have stored with tsconfig.
     */
    switch (GetCreatorID()) {
    case 'Tsc3': cat = prefs.launch1cat; break;
    case 'Tsc4': cat = prefs.launch2cat; break;
    case 'Tsc5': cat = prefs.launch3cat; break;
    default: cat = 0xff; break; /* All */
    }
    return cat;
}

void launch_launcher(UInt8 category)
{
    Int16 prefsVersion;
    UInt16 prefsSize;
    EventType e;
    UInt8 lnch1[8]; /* it's really on 4 bytes */

    /*
     * This is pref #1 from the launcher.  Offset 1 appears to be the category
     * which is used on the next invocation of the launcher.  So I set it prior
     * to enqueueing the launch char.
     */
    prefsSize = sizeof(lnch1);
    prefsVersion = PrefGetAppPreferences('lnch', 1 /* Pref ID */,
        &lnch1, &prefsSize, false /* unsaved prefs */);

    if (prefsVersion != noPreferenceFound) {
        lnch1[1] = category;
        PrefSetAppPreferences('lnch', 1 /* Pref ID */, prefsVersion,
            &lnch1, prefsSize, false /* unsaved prefs */);
    }

    e.eType = keyDownEvent;
    e.data.keyDown.chr = launchChr;
    e.data.keyDown.modifiers = commandKeyMask;
    EvtAddEventToQueue(&e);
}

/* The main event loop */
void EventLoop(void)
{
    Err err;
    EventType e;
    Int32 eventTimeout;

    eventTimeout = evtWaitForever;

    do {

    EvtGetEvent(&e, eventTimeout);

    if (!SysHandleEvent(&e)) {
        if (!MenuHandleEvent(NULL, &e, &err)) {
            FrmDispatchEvent(&e);
        }
    }

    } while (e.eType != appStopEvent);

}

/*
 * The program starts here.
 */
UInt32 PilotMain(UInt16 cmd, void *cmdPBP, UInt16 launchFlags)
{
    Err err;
    UInt32 romVersion;
    UInt32 theInstalledRAM; 

    switch (cmd) {
    case sysAppLaunchCmdNormalLaunch:
        /*
         * This ROM version test comes from Palm's Address application.
         * Thankfully too because I would hate to guess at this kind of
         * stuff and make some poor guy's Pilot blow up.
         */
        FtrGet(sysFtrCreator, sysFtrNumROMVersion, &romVersion);
        /*
         * Need at least a PalmPilot Pro because of the TCP/IP.
         */
        if (romVersion < 0x03000000) {
            if ((launchFlags &
                (sysAppLaunchFlagNewGlobals | sysAppLaunchFlagUIApp)) ==
                (sysAppLaunchFlagNewGlobals | sysAppLaunchFlagUIApp)) {
                /*
                 * Pilot 1.0 will continuously relaunch this app unless we
                 * switch to another safe one.
                 */
                if (romVersion < 0x02000000) {
                    FrmAlert(RomIncompatibleAlert);
                    AppLaunchWithCommand(sysFileCDefaultApp,
                        sysAppLaunchCmdNormalLaunch, NULL);
                }
                err = MemCardInfo( 0, NULL, NULL, NULL, NULL, NULL,
                    &theInstalledRAM, NULL); 
                if (!err && (theInstalledRAM == 524288)) { 
                    /*
                     * We are running on a PalmPilot Personal unit
                     */
                    FrmAlert(RomIncompatibleAlert);
                    return sysErrRomIncompatible;
                }
            }
        }
        LoadPrefs(&prefs);
        launch_launcher(get_proper_category());
        EventLoop();
        break;

    default:
        return sysErrParamErr;
    }

    return 0;
}
