/* $Id: tsphone.c,v 1.5 2003/04/15 00:47:07 chrisf Exp $ */

/*
TreoScreens - Select default screen for phone etc.
Copyright (C) 2003 Chris Faherty

This program is free software; you can redistribute it and/or
modify it under the terms of the GNU General Public License
as published by the Free Software Foundation; either version 2
of the License, or (at your option) any later version.

This program is distributed in the hope that it will be useful,
but WITHOUT ANY WARRANTY; without even the implied warranty of
MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
GNU General Public License for more details.

You should have received a copy of the GNU General Public License
along with this program; if not, write to the Free Software
Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.
*/

#include <PalmOS.h>
#include <Handspring/HsExt.h>
#include <Handspring/HsAppLaunchCmd.h>
#include "tsprefs.h"
#include "tsconfig.h"
#include "tsphoneRsc.h"
#include "tsphone.h"

/* globals */

/* prefs */
struct sprefs prefs;

Err launch_phone(UInt16 phoneapplaunch)
{
    UInt16 cardNo;
    LocalID dbID;
    DmSearchStateType SearchState;
    Err err;
    UInt16 cmd;

    err = DmGetNextDatabaseByTypeCreator(true, &SearchState,
        sysFileTApplication, hsFileCPhone, true, &cardNo, &dbID);
    if (err) return err;

    switch (phoneapplaunch) {
    case 0: /* Speed Dial */
        cmd = phoneAppLaunchCmdViewSpeed;
        break;
    case 1: /* Keypad */
        cmd = phoneAppLaunchCmdViewKeypad;
        break;
    case 2: /* Call History */
        cmd = phoneAppLaunchCmdViewHistory;
        break;
    default:
        cmd = sysAppLaunchCmdNormalLaunch;
        break;
    }

    err = SysUIAppSwitch(cardNo, dbID, cmd, NULL);
    return err;
}

/*
 * The program starts here.
 */
UInt32 PilotMain(UInt16 cmd, void *cmdPBP, UInt16 launchFlags)
{
    Err err;
    UInt32 romVersion;
    UInt32 theInstalledRAM; 

    switch (cmd) {
    case sysAppLaunchCmdNormalLaunch:
        /*
         * This ROM version test comes from Palm's Address application.
         * Thankfully too because I would hate to guess at this kind of
         * stuff and make some poor guy's Pilot blow up.
         */
        FtrGet(sysFtrCreator, sysFtrNumROMVersion, &romVersion);
        /*
         * Need at least a PalmPilot Pro because of the TCP/IP.
         */
        if (romVersion < 0x03000000) {
            if ((launchFlags &
                (sysAppLaunchFlagNewGlobals | sysAppLaunchFlagUIApp)) ==
                (sysAppLaunchFlagNewGlobals | sysAppLaunchFlagUIApp)) {
                /*
                 * Pilot 1.0 will continuously relaunch this app unless we
                 * switch to another safe one.
                 */
                if (romVersion < 0x02000000) {
                    FrmAlert(RomIncompatibleAlert);
                    AppLaunchWithCommand(sysFileCDefaultApp,
                        sysAppLaunchCmdNormalLaunch, NULL);
                }
                err = MemCardInfo( 0, NULL, NULL, NULL, NULL, NULL,
                    &theInstalledRAM, NULL); 
                if (!err && (theInstalledRAM == 524288)) { 
                    /*
                     * We are running on a PalmPilot Personal unit
                     */
                    FrmAlert(RomIncompatibleAlert);
                    return sysErrRomIncompatible;
                }
            }
        }
        LoadPrefs(&prefs);
        return launch_phone(prefs.phoneapplaunch);

    default:
        return sysErrParamErr;
    }

    return 0;
}
